import Foundation
import GRDB

extension Database {
    static var migrator: DatabaseMigrator = {
        var migrator = DatabaseMigrator()

        // flush db on schema change (only in DEV mode)
        #if DEBUG
            migrator.eraseDatabaseOnSchemaChange = true
        #endif

        // 1st migration - basic tables
        migrator.registerMigration("Add basic tables") { db in
            // accounts
            try db.create(table: "accounts", options: [.ifNotExists]) { table in
                table.column("bareJid", .text).notNull().primaryKey().unique(onConflict: .replace)
                table.column("pass", .text).notNull()
                table.column("isActive", .boolean).notNull().defaults(to: true)
            }

            // rosters
            try db.create(table: "rosterVersions", options: [.ifNotExists]) { table in
                table.column("bareJid", .text).notNull().primaryKey().unique(onConflict: .replace)
                table.column("version", .text).notNull()
            }
            try db.create(table: "rosters", options: [.ifNotExists]) { table in
                table.column("bareJid", .text).notNull()
                table.column("contactBareJid", .text).notNull()
                table.column("name", .text)
                table.column("subscription", .text).notNull()
                table.column("ask", .boolean).notNull().defaults(to: false)
                table.column("data", .text).notNull()
                table.primaryKey(["bareJid", "contactBareJid"], onConflict: .replace)
                table.column("locallyDeleted", .boolean).notNull().defaults(to: false)
            }

            // chats
            try db.create(table: "chats", options: [.ifNotExists]) { table in
                table.column("id", .text).notNull().primaryKey().unique(onConflict: .replace)
                table.column("account", .text).notNull()
                table.column("participant", .text).notNull()
                table.column("type", .integer).notNull()
            }

            // messages
            try db.create(table: "messages", options: [.ifNotExists]) { table in
                table.column("id", .text).notNull().primaryKey().unique(onConflict: .replace)
                table.column("chatId", .text).notNull().references("chats", onDelete: .cascade)
                table.column("fromJid", .text).notNull()
                table.column("toJid", .text).notNull()
                table.column("timestamp", .datetime).notNull()
                table.column("body", .text)
                // table.column("isReaded", .boolean).notNull().defaults(to: false)
                // table.column("subject", .text)
                // table.column("threadId", .text)
                // table.column("errorType", .text)
            }
        }

        // return migrator
        return migrator
    }()
}
