import Combine
import Foundation
import GRDB

final class ChatsService: ObservableObject {
    private weak var database: DatabaseQueue?
    private var cancellables: Set<AnyCancellable> = []

    @Published private(set) var chats: [Chat] = []

    init(_ db: DatabaseQueue) {
        database = db
        doSubscriptions()
    }

    private func doSubscriptions() {
        // watch chats
        guard let db = database else { return }
        ValueObservation
            .tracking(Chat.fetchAll)
            .publisher(in: db, scheduling: .immediate)
            .sink { _ in
                // Handle completion
            } receiveValue: { chats in
                self.chats = chats
            }
            .store(in: &cancellables)
    }

    // get existing chat or create a new one
    func getChat(_ account: String, _ participant: String) -> Chat? {
        guard let db = database else { return nil }
        if let chat = chats.first(where: { $0.account == account && $0.participant == participant }) {
            return chat
        } else {
            let chat = Chat(
                id: UUID().uuidString,
                account: account,
                participant: participant,
                type: .chat
            )
            do {
                try db.write { db in
                    try chat.save(db)
                }
            } catch {
                logIt(.error, "Failed to create chat: \(error)")
            }
            return chat
        }
    }
}
