import Combine
import Foundation
import GRDB
import Martin
import SwiftUI

final class ConversationService: ObservableObject {
    let account: Account
    let chat: Chat
    let roster: Roster
    private var cancellables: Set<AnyCancellable> = []

    weak var database: DatabaseQueue?
    weak var client: XMPPClient?

    @Published var messages: [Message] = []

    init(account: Account, chat: Chat, roster: Roster, db: DatabaseQueue, client: XMPPClient) {
        self.account = account
        self.chat = chat
        self.roster = roster
        database = db
        self.client = client

        subscribe()
    }

    func subscribe() {
        guard let db = database else { return }
        ValueObservation
            .tracking { _db in
                try Message
                    .filter(Column("chatId") == self.chat.id)
                    .order(Column("timestamp").desc)
                    .fetchAll(_db)
            }
            .publisher(in: db, scheduling: .immediate)
            .sink { _ in
                // Handle completion
            } receiveValue: { [weak self] messages in
                self?.messages = messages
            }
            .store(in: &cancellables)
    }
}
