import Combine
import Foundation
import GRDB
import Martin

final class MessageService: ObservableObject {
    private weak var database: DatabaseQueue?
    private var cancellables: Set<AnyCancellable> = []

    var activeChat: Chat?

    init(_ db: DatabaseQueue) {
        database = db
    }

    func updateSubscriptions(for clients: [XMPPClient]) {
        cancellables = Set<AnyCancellable>()
        for client in clients {
            client.module(MessageModule.self).messagesPublisher
                .sink { [weak self] message in
                    self?.processMessage(message.message, message.chat)
                }
                .store(in: &cancellables)
        }
    }

    private func processMessage(_ message: Martin.Message, _ chat: Martin.ChatProtocol) {
        print("\n---Message received: \(message.debugDescription) from \(message.from)\n--Chat: \(chat)\n")

        // no store hint if message is drawing
        // need a way to show it on the chat (three animated dots, probably)
        if message.hints.contains(.noStore) {
            return
        }

        // process error messages
        if let error = message.error {
            print("Error message: \(error)")
            return
        }

        guard let database else { return }
        guard let id = message.id else { print("--message: no id"); return }
        guard let from = message.from else { print("--message: no from"); return }
        guard let to = message.to else { print("--message: no to"); return }

        let message = Message(
            id: id,
            chatId: "???",
            fromJid: from.stringValue,
            toJid: to.stringValue,
            timestamp: Date(),
            body: message.body
        )

        do {
            try database.write { db in
                try message.save(db)
            }
        } catch {
            print("Error saving message: \(error)")
        }
    }
}
