import SwiftUI

struct SharedTabBar: View {
    var body: some View {
        VStack(spacing: 0) {
            Rectangle()
                .frame(maxWidth: .infinity)
                .frame(height: 0.2)
                .foregroundColor(.Main.separator)
            HStack(spacing: 0) {
                SharedTabBarButton(buttonFlow: .contactsList)
                SharedTabBarButton(buttonFlow: .chatsList)
                SharedTabBarButton(buttonFlow: .settings)
            }
            .background(Color.Main.backgroundDark)
        }
        .frame(height: 50)
    }
}

private struct SharedTabBarButton: View {
    @EnvironmentObject var state: AppState

    let buttonFlow: AppFlow

    var body: some View {
        ZStack {
            VStack(spacing: 2) {
                buttonImg
                    .foregroundColor(buttonFlow == state.flow ? .Tango.orangeMedium : .Main.gray)
                    .font(.system(size: 24, weight: .light))
                    .symbolRenderingMode(.hierarchical)
                Text(buttonTitle)
                    .font(.sub1)
                    .foregroundColor(buttonFlow == state.flow ? .Main.black : .Main.gray)
            }
            Rectangle()
                .foregroundColor(.white.opacity(0.01))
                .onTapGesture {
                    state.flow = buttonFlow
                }
        }
    }

    var buttonImg: Image {
        switch buttonFlow {
        case .contactsList:
            return Image(systemName: "person.2.fill")

        case .chatsList:
            return Image(systemName: "bubble.left.fill")

        case .settings:
            return Image(systemName: "gearshape.fill")

        default:
            return Image(systemName: "questionmark.circle")
        }
    }

    var buttonTitle: String {
        switch buttonFlow {
        case .contactsList:
            return "Contacts"

        case .chatsList:
            return "Chats"

        case .settings:
            return "Settings"

        default:
            return "Unknown"
        }
    }
}
