import SwiftUI

struct ChatsListScreen: View {
    @EnvironmentObject var state: AppState

    var body: some View {
        ZStack {
            // Background color
            Color.Main.backgroundLight
                .ignoresSafeArea()

            // Content
            VStack(spacing: 0) {
                // Header
                ChatsScreenHeader()

                // Chats list
                if !state.chats.isEmpty {
                    List {
                        ForEach(state.chats) { chat in
                            ChatsRow(chat: chat)
                        }
                    }
                    .listStyle(.plain)
                    .background(Color.Main.backgroundLight)
                } else {
                    Spacer()
                }

                // Tab bar
                SharedTabBar()
            }
        }
    }
}

private struct ChatsScreenHeader: View {
    var body: some View {
        ZStack {
            // bg
            Color.Main.backgroundDark
                .ignoresSafeArea()

            // title
            Text(L10n.Chats.title)
                .font(.head2)
                .foregroundColor(Color.Main.black)

            // HStack {
            //     Spacer()
            //     Image(systemName: "plus")
            //         .foregroundColor(Color.Tango.orangeMedium)
            //         .tappablePadding(.symmetric(12)) {
            //             print("Add contact")
            //         }
            // }
            // .padding(.horizontal, 16)
        }
        .frame(height: 44)
    }
}

private struct ChatsRow: View {
    @EnvironmentObject var state: AppState

    var chat: Chat

    var body: some View {
        VStack(spacing: 0) {
            HStack(spacing: 8) {
                ZStack {
                    Circle()
                        .frame(width: 44, height: 44)
                        .foregroundColor(.red)
                    Text(chat.participant.firstLetter)
                        .foregroundColor(.white)
                        .font(.body1)
                }
                Text(chat.participant)
                    .foregroundColor(Color.Main.black)
                    .font(.body2)
                Spacer()
            }
            .padding(.horizontal, 16)
            .padding(.vertical, 4)
            Rectangle()
                .frame(maxWidth: .infinity)
                .frame(height: 1)
                .foregroundColor(.Main.backgroundDark)
        }
        .sharedListRow()
        .onTapGesture {
            state.startChat(chat)
        }
    }
}
